@@ -2,33 +2,37 @@
 
 import os
 import pkgutil
+from lambda_toolkit.modules.utils import Utils
 from shutil import make_archive
 from shutil import rmtree
-from urllib import urlretrieve
 from zipfile import ZipFile
+import sys
 import time
-import boto3
-import logger
-
 
 class Project:
     def __init__(self, conf, kwargs):
-        self.log = logger.get_my_logger(self.__class__.__name__)
-        self.conf = conf
+        self.log = conf.log
+
+        if Utils.check_kwargs(kwargs, "region"):
+            self.log.info("Updating region to '" + kwargs['region'] + "'.")
+            self.conf = conf.set_region(kwargs['region'])
+        else:
+            self.conf = conf
+
+        if Utils.check_kwargs(kwargs, "projectname"):
+            self.projectname = kwargs['projectname']
+            self._set_project(self.projectname)
+
         self.kwargs = kwargs
         self.projects = self.conf.projects.keys()
-        if 'projectname' in kwargs and kwargs['projectname'] is not None:
-            self._set_project(kwargs['projectname'])
 
 
     def _update_new_region(self, reg):
-        self.conf.region = reg
-        self.conf._init_confs()
+        self.conf.set_region(reg)
         self.projects = self.conf.projects.keys()
 
-
     def import_all_project(self):
-        lambdas = self.conf.get_boto3("lambda").list_functions()
+        lambdas = self.conf.get_boto3("lambda", "client").list_functions()
         for mylb in lambdas['Functions']:
             if mylb['FunctionName'] not in self.conf.proxies:
                 self._set_project(mylb['FunctionName'])
@@ -38,7 +42,7 @@ class Project:
         return self.conf
 
     def deploy_all_project(self):
-        for s in self.projects:
+        for s in list(self.projects):
             self._set_project(s)
             self.deploy_project()
 
@@ -46,7 +50,7 @@ class Project:
         return self.conf
 
     def undeploy_all_project(self):
-        for s in self.projects:
+        for s in list(self.projects):
             self._set_project(s)
             self.undeploy_project()
 
@@ -61,10 +65,10 @@ class Project:
 
             if 'python' in self.kwargs['runtime']:
                 open(self.project_dir + "/__init__.py", 'a').close()
-                with open(os.path.join(self.project_dir, "index.py"), "w") as text_file:
+                with open(os.path.join(self.project_dir, "index.py"), "wb") as text_file:
                     text_file.write(pkgutil.get_data("lambda_toolkit", self.conf.sett['C_LAMBDASTANDARD_FUNC_PY']))
             elif 'nodejs' in self.kwargs['runtime']:
-                with open(os.path.join(self.project_dir, "index.js"), "w") as text_file:
+                with open(os.path.join(self.project_dir, "index.js"), "wb") as text_file:
                     text_file.write(pkgutil.get_data("lambda_toolkit", self.conf.sett['C_LAMBDASTANDARD_FUNC_JS']))
 
             self.log.info("Project '" + self.projectname + "' "
@@ -76,7 +80,7 @@ class Project:
         return self.conf
 
     def delete_all_project(self):
-        for s in self.projects:
+        for s in list(self.projects):
             self._set_project(s)
             self.delete_project()
 
@@ -102,7 +106,7 @@ class Project:
             return self.conf
 
         try:
-            lambda_function = self.conf.get_boto3("lambda").get_function(FunctionName=self.projectname)
+            lambda_function = self.conf.get_boto3("lambda", "client").get_function(FunctionName=self.projectname)
 
             if self.projectname in self.conf.projects:
                 self.log.info("Project '" + self.projectname + "' already exists in your configuration. Updating.")
@@ -115,11 +119,18 @@ class Project:
                 self.conf.projects[self.projectname]['runtime'] = lambda_function['Configuration']['Runtime']
                 self.log.info("Project " + self.projectname + " imported.")
 
-            urlretrieve(lambda_function['Code']['Location'], self.project_zip_file)
-            zip_ref = ZipFile(self.project_zip_file, 'r')
-            zip_ref.extractall(self.project_dir)
-            zip_ref.close()
+            import requests, zipfile
+            if sys.version_info[0] == 3:
+                import io
+                r = requests.get(lambda_function['Code']['Location'], stream=True)
+                z = zipfile.ZipFile(io.BytesIO(r.content))
+            else:
+                import StringIO
+                r = requests.get(lambda_function['Code']['Location'], stream=True)
+                z = zipfile.ZipFile(StringIO.StringIO(r.content))
 
+            z.extractall(self.project_dir)
+            z.close()
         except Exception as e:
             self.log.error(str(e))
             self.log.warn("The project '" + self.projectname + "' does not exist in AWS environment.")
@@ -133,20 +144,20 @@ class Project:
         make_archive(self.project_zip_file_without_ext, "zip", self.project_dir)
 
         try:
-            self.conf.get_boto3("lambda").get_function(FunctionName=self.projectname)
+            self.conf.get_boto3("lambda", "client").get_function(FunctionName=self.projectname)
             replace = True
         except Exception:
             replace = False
 
         try:
             if replace:
-                self.conf.get_boto3("lambda").update_function_code(
+                self.conf.get_boto3("lambda", "client").update_function_code(
                     FunctionName=self.projectname,
                     ZipFile=open(self.project_zip_file, "rb").read()
                 )
                 self.log.info("Lambda project " + self.projectname + " was redeployed.")
             else:
-                self.conf.get_boto3("lambda").create_function(
+                self.conf.get_boto3("lambda", "client").create_function(
                     FunctionName=self.projectname,
                     Runtime=self.conf.projects[self.projectname]['runtime'],
                     Role=self.kwargs['rolename'],
@@ -172,7 +183,7 @@ class Project:
             self.log.critical("Project '" + self.projectname + "' does not exist.")
 
         try:
-            self.conf.get_boto3("lambda").delete_function(FunctionName=self.projectname)
+            self.conf.get_boto3("lambda", "client").delete_function(FunctionName=self.projectname)
             self.log.info("Project '" + self.projectname + "' is now undeployed.")
         except Exception as e:
             self.log.warn("Project '" + self.projectname + "' is not deployed.")
@@ -210,18 +221,25 @@ class Project:
 
         return self.conf
 
-    def list_all_aws_project(self):
+    def list_aws_all_project(self):
         for reg in self.conf.aws_regions:
             self._update_new_region(reg)
             self.list_aws_project()
 
         return self.conf
 
+    def list_all_project(self):
+        for reg in self.conf.aws_regions:
+            self._update_new_region(reg)
+            self.list_project()
+
+        return self.conf
+
     def check_regions_delay_project(self):
         for reg in self.conf.aws_regions:
             self._update_new_region(reg)
             start_time = time.time()
-            self.conf.get_boto3("lambda").list_functions()
+            self.conf.get_boto3("lambda", "client").list_functions()
             self.log.info('{0: <{1}}'.format(self.conf.region + ": ", 20) +
                           str((time.time() - start_time)))
 
@@ -229,7 +247,7 @@ class Project:
 
     def list_aws_project(self):
         self.log.info("AWS Projects (Lambda Functions in " + self.conf.region + "):")
-        lambdas = self.conf.get_boto3("lambda").list_functions()
+        lambdas = self.conf.get_boto3("lambda", "client").list_functions()
         for mylb in lambdas['Functions']:
             imported = False
             if mylb['FunctionName'] in self.projects:
@@ -245,7 +263,7 @@ class Project:
 
     def list_project(self):
         if len(self.projects) > 0:
-            self.log.info("User Projects (Lambda Functions):")
+            self.log.info("User Projects (Lambda Functions in " + self.conf.region + "):")
             for p in self.projects:
                 self.log.info('{0: <{1}}'.format("- Project:", 15) +
                               '{0: <{1}}'.format(p, 25) +
@@ -269,12 +287,15 @@ class Project:
     def _set_project(self, projectname):
         self.log.debug("Updating project environment to: '" + projectname + "'")
         if projectname in self.conf.proxies.keys():
-            self.log.critical("You cannot create a project with the same name of an existing proxy.")
+            self.log.critical("You cannot act in a project with the same name of an existing proxy.")
         self.projectname = projectname
-        projectname_region = self.conf.region + "/" + projectname
-        self.project_dir = os.path.join(self.conf.lambdas_dir, projectname_region)
-        self.project_zip_dir = os.path.join(self.conf.lambdas_dir,
-                                            self.conf.region + "/" + self.conf.sett['C_LAMBDAS_ZIP_DIR'])
+        self.project_dir = os.path.join(os.path.expanduser(self.conf.sett['C_BASE_DIR']),
+                                        self.conf.sett['C_LAMBDAS_DIR'],
+                                        self.conf.region, projectname)
+        self.project_zip_dir = os.path.join(os.path.expanduser(self.conf.sett['C_BASE_DIR']),
+                                            self.conf.sett['C_LAMBDAS_DIR'],
+                                            self.conf.region,
+                                            self.conf.sett['C_LAMBDAS_ZIP_DIR'])
         self.project_zip_file = os.path.join(self.project_zip_dir, projectname + ".zip")
         self.project_zip_file_without_ext = os.path.join(self.project_zip_dir, projectname)
 
